'' =================================================================================================
''
''   File....... jm_shared_hub.spin
''   Purpose....
''   Author..... Jon McPhalen
''               Copyright (c) 2022 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 22 SEP 2022
''
''   {$P1}
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { terminal }

  BR_TERM  = 115_200                                            ' terminal baud rate

  #0, T_PST, T_ANSI                                             ' terminal types

  T_TYPE = T_PST


con { fixed io pins }

  RX1      = 31  { I }                                          ' serial / programming
  TX1      = 30  { O }

  SDA1     = 29  { I/O }                                        ' i2c / eeprom
  SCL1     = 28  { I/O }


con { app io pins }


con

  #true,  ON, OFF
  #false, NO, YES


obj

' main                                                          ' * master Spin cog
  time : "jm_time_80"                                           '   timing and delays (80MHz)
  io   : "jm_io"                                                '   simple io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
  ansi : "jm_ansi"                                              '   ANSI terminal control sequences

' * uses cog when loaded


dat { pre-initialized }


var { globals }


pub main | x, y

  setup

  wait_for_terminal(true)
  time.pause(100)

  a_start
  b_start(a_counter_addr)

  x := -1

  repeat
    y := a_get_count
    if (y <> x)
      term.dec(y)
      term.tx(13)
      x := y


pub setup

'' Setup IO and objects for application

  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)

  term.tstart(BR_TERM)                                          ' start serial for terminal *


pub wait_for_terminal(clear)

'' Wait for terminal to be open and key pressed

  term.rxflush
  term.rx
  if (clear)
    clear_screen


pub clear_screen

  if (T_TYPE == T_PST)
    term.tx(term#CLS)
  else
    term.str(ansi.hide_cursor)
    term.str(ansi.home)
    term.str(ansi.cls)


con { object a }

var

  long  counter


pub a_start

  cognew(@coga, a_counter_addr)                         ' pass counter address to pasm cog


pub a_get_count

  return counter                                        ' return counter value to parent


pub a_counter_addr

  return @counter                                       ' return counter hub address to parent


dat { object a pasm }

                        org       0

coga                    rdlong    t1, par                       ' get current count
                        adds      t1, #1                        ' increment
                        wrlong    t1, par                       ' write back

                        mov       t1, cnt
                        add       t1, DELAY
                        waitcnt   t1, #0
                        jmp       #coga


DELAY                   long      CLK_FREQ >> 2                 ' 1/4 second

t1                      res       1                             ' work var


con { object b }

pub b_start(shared)

  cognew(@cogb, shared)


dat { object b pasm }

                        org       0

cogb                    mov       mask, #%11                    ' two pins
                        shl       mask, #26                     ' use P26 and P27 (LEDs on PAB/FLiP)
                        or        dira, mask                    ' set to output

                        neg       last, #1                      ' preset to force initial change

cogb_loop               rdlong    t2, par                       ' read count
                        cmps      t2, last              wc, wz  ' compare with last
        if_e            jmp       #cogb_loop                    ' wait for change

                        mov       last, t2                      ' save new count

                        andn      outa, mask                    ' clear LEDs
                        shl       t2, #26                       ' align counter lsb
                        and       t2, mask                      ' trim to 2 bits
                        or        outa, t2                      ' update LEDs

                        jmp       #cogb_loop


mask                    res       1                             ' leds mask
last                    res       1                             ' last count value

t2                      res       1                             ' work var

                        fit       496


con { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}